﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media;

namespace GameLoop_CompositionTarget
{
	public class GameLoop
	{
		/// <summary>
		/// 마지막으로 기록된 시간
		/// </summary>
		protected DateTime m_datetimeLastTick;

		/// <summary>
		/// 게임 루프 발생용 핸들러
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="elapsed"></param>
		public delegate void UpdateHandler(object sender, TimeSpan elapsed);
		/// <summary>
		/// 게임 루프가 동작하면 발생 합니다.
		/// </summary>
		public event UpdateHandler OnUpdate;


		/// <summary>
		/// 현재시간 기록.
		/// -반복해서 호출되기 때문에 가비지컬랙터를 거치치지 않게 하여 성능을 늘리기 위해 선언함
		/// </summary>
		private DateTime m_datetimeNow;
		/// <summary>
		/// 경과 시간
		/// -반복해서 호출되기 때문에 가비지컬랙터를 거치치지 않게 하여 성능을 늘리기 위해 선언함
		/// </summary>
		private TimeSpan m_timespanElapsed;

		/// <summary>
		/// 연결된 이벤트의 카운트
		/// </summary>
		public int EventCount = 0;


		public void Tick()
		{
			//지금 시간 기록
			m_datetimeNow = DateTime.Now;

			//경과시간 = 지금시간 - 마지막 시간
			m_timespanElapsed = m_datetimeNow - m_datetimeLastTick;
			//마지막 시간 기록
			m_datetimeLastTick = m_datetimeNow;

			//업데이터 이벤트가 연결 되어 있나?
			if (OnUpdate != null)
			{
				//되어 있으면 동작시킨다.
				OnUpdate(this, m_timespanElapsed);
			}
		}

		void CompositionTarget_Rendering(object sender, EventArgs e)
		{
			Tick();
		}

		/// <summary>
		/// 게임루프를 실행합니다.
		/// 무한루프 형태로 진행 됩니다.
		/// 베이스의 Start()를 오버라이드했기 때문에 계속 자신이 호출되는 효과가 있습니다.
		/// </summary>
		public void Start()
		{
			//연결된 이벤트가 있는가?
			if (0 < EventCount)
			{
				//연결된 이벤트가 있다.
				//있으면 동작을 막는다. 
				return;
			}

			//틱호출 이벤트를 연결해줌
			CompositionTarget.Rendering += new EventHandler(CompositionTarget_Rendering);
			//이벤트가 추가 되었음을 표시
			++EventCount;

			//마지막 틱 기록
			m_datetimeLastTick = DateTime.Now;

			//베이스에 스타트를 실행함
			Start();
		}

		/// <summary>
		/// 게임루프를 정시 시킵니다.
		/// </summary>
		public void Stop()
		{
			//연결된 이벤트가 있는지?
			if (0 >= EventCount)
			{
				//연결된 이벤트가 없다.
				//없으면 처리할게 없다.
				return;
			}

			//틱호출 이벤트를 끊습니다.
			CompositionTarget.Rendering -= new EventHandler(CompositionTarget_Rendering);
			//이벤트가 제거 되었음을 표시
			--EventCount;
			//베이스에 스타트를 종료함
			Stop();
		}
	}
}
